From a47f1b72ea469ff295cf083bb108003d5460dc39 Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa@rosenzweig.io>
Date: Sat, 17 Dec 2022 16:28:54 -0500
Subject: [PATCH 1/2] asahi: Use writeback when it looks beneficial

When playing the My Little Pony theme song at 1080p on T8103, with mpv's GPU
compositing but software decoding, CPU usage drops from 200% to 50% due to
proper caching of the staging resource.

Signed-off-by: Alyssa Rosenzweig <alyssa@rosenzweig.io>
---
 src/gallium/drivers/asahi/agx_pipe.c | 14 +++++++++++++-
 1 file changed, 13 insertions(+), 1 deletion(-)

diff --git a/src/gallium/drivers/asahi/agx_pipe.c b/src/gallium/drivers/asahi/agx_pipe.c
index debc816912e..14233aea962 100644
--- a/src/gallium/drivers/asahi/agx_pipe.c
+++ b/src/gallium/drivers/asahi/agx_pipe.c
@@ -533,7 +533,19 @@ agx_resource_create_with_modifiers(struct pipe_screen *screen,
       (bind & PIPE_BIND_SHADER_IMAGE) ? "Shader image" :
       "Other resource";
 
-   nresource->bo = agx_bo_create(dev, nresource->layout.size_B, 0, label);
+   uint32_t create_flags = 0;
+
+   /* Default to write-combine resources, but use writeback if that is expected
+    * to be beneficial.
+    */
+   if (nresource->base.usage == PIPE_USAGE_STAGING ||
+       (nresource->base.flags & PIPE_RESOURCE_FLAG_MAP_COHERENT)) {
+
+      create_flags |= AGX_BO_CREATE_WRITEBACK;
+   }
+
+   nresource->bo = agx_bo_create(dev, nresource->layout.size_B,
+                                 create_flags, label);
 
    if (!nresource->bo) {
       FREE(nresource);
-- 
2.35.1

